from .registry import RegistryTestCase
from .class_decorator import ClassDecoratorTestCase
from .autodiscover import AutodiscoverTestCase
from .views import ViewsTestCase
from .decorators import DecoratorsTestCase
from .shortcuts import ShortcutsTestCase
